package com.sugamobile.realwear_dictation

import android.app.Activity
import android.content.Intent
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.PluginRegistry

class RealwearDictationPlugin : FlutterPlugin, MethodCallHandler, ActivityAware, PluginRegistry.ActivityResultListener {
    companion object {
        private const val TAG = "RealwearDictationPlugin"

        private const val REQUEST_CODE_DICTATION_TEXT = 2001
        private const val REQUEST_CODE_DICTATION_NUMERIC = 2002

        private const val INTENT_ACTION_DICTATION = "com.realwear.keyboard.intent.action.DICTATION"

        private const val RECEIVING_EVENT_OPEN_DICTATION = "open_dictation"

        private const val SENDING_EVENT_DICTATION_RESULT = "dictation_result"
    }

    private lateinit var channel: MethodChannel

    private var activity: Activity? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "realwear_dictation")
        channel.setMethodCallHandler(this)
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
        binding.addActivityResultListener(this)
    }

    override fun onDetachedFromActivity() {
        activity = null
    }

    override fun onDetachedFromActivityForConfigChanges() {
        activity = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        activity = binding.activity
        binding.addActivityResultListener(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
        when (call.method) {
            RECEIVING_EVENT_OPEN_DICTATION -> {
                val inputType = call.argument<String>("input_type")!!
                openDictation(inputType)
                result.success(null)
            }
            else -> result.notImplemented()
        }
    }

    private fun openDictation(inputType: String) {
        val intent = Intent(INTENT_ACTION_DICTATION)
        val requestCode = if (inputType == "text") REQUEST_CODE_DICTATION_TEXT else REQUEST_CODE_DICTATION_NUMERIC
        activity?.startActivityForResult(intent, requestCode)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        when (requestCode) {
            REQUEST_CODE_DICTATION_TEXT, REQUEST_CODE_DICTATION_NUMERIC -> {
                val inputType = if (requestCode == REQUEST_CODE_DICTATION_TEXT) "text" else "numeric"
                var result: String? = null
                if (resultCode == Activity.RESULT_OK && data != null) {
                    result = data.getStringExtra("result")
                }
                channel.invokeMethod(SENDING_EVENT_DICTATION_RESULT, mapOf("result" to result, "input_type" to inputType))
                return true
            }
        }
        return false
    }
}
