import 'dart:async';

import 'package:event_bus/event_bus.dart';
import 'package:flutter/services.dart';
import 'package:get/get.dart';
import 'package:logger/logger.dart';
import 'package:realwear_dictation/core/helpers.dart';
import 'package:realwear_dictation/enums/dictation_input_type.dart';
import 'package:realwear_dictation/events/dictation_result_event.dart';
import 'package:realwear_dictation/ui/dictation_input_dialog.dart';
import 'package:suga_core/suga_core.dart';

class RealwearDictation {
  bool _initialized = false;
  final _methodChannel = const MethodChannel('realwear_dictation');

  final Logger _logger;
  final EventBus _eventBus;
  final bool _debugMode;

  static const String _receivingEventDictationResult = 'dictation_result';

  static const String _sendingEventOpenDictation = 'open_dictation';

  RealwearDictation(this._logger, this._eventBus, this._debugMode);

  void init() {
    if (_initialized) {
      throw Exception('RealwearDictation already initialized');
    }
    _initialized = true;

    _methodChannel.setMethodCallHandler(
      (call) async {
        _logger.d('RealwearDictation MethodCall: $call');
        switch (call.method) {
          case _receivingEventDictationResult:
            final arguments = Map<String, String?>.from(call.arguments as Map);
            _onDictationResult(
              arguments['result'],
              dictationInputTypeFromValue(arguments['input_type']) ?? DictationInputType.text,
            );
            break;
        }
      },
    );
  }

  Future<Unit> openDictation({required DictationInputType inputType}) async {
    if (!_debugMode) {
      await _methodChannel.invokeMethod(
        _sendingEventOpenDictation,
        {
          'input_type': inputType.getValue(),
        },
      );
    } else {
      unawaited(
        Get.dialog(
          DictationInputDialog(
            onResult: (result) => _onDictationResult(result, inputType),
          ),
        ),
      );
    }
    return unit;
  }

  void _onDictationResult(String? result, DictationInputType inputType) {
    switch (inputType) {
      case DictationInputType.text:
        _eventBus.fire(DictationResultEvent(result));
        break;
      case DictationInputType.numeric:
        _eventBus.fire(DictationResultEvent(parseDictationResultToNumeric(result)));
        break;
    }
  }
}
